package com.celestek.hexcraft.block;

import com.celestek.hexcraft.HexCraft;
import com.celestek.hexcraft.client.HexClientProxy;
import cpw.mods.fml.relauncher.Side;
import cpw.mods.fml.relauncher.SideOnly;
import net.minecraft.block.Block;
import net.minecraft.block.material.Material;
import net.minecraft.world.IBlockAccess;
import net.minecraft.world.World;
import net.minecraftforge.common.util.ForgeDirection;

public class HexBlock extends Block {

    // HEXCraft's ID for the block.
    private final int idBlock;

    /**
     * Constructor for the block.
     * @param material Default material to use.
     */
    public HexBlock(Material material) {
        super(material);
        // Save the current HexCraft block ID.
        this.idBlock = HexCraft.idCounter;

        // Increment block counter in HexCraft class.
        HexCraft.idCounter++;
    }

    /**
     * Forces the usage of custom renderer.
     */
    @Override
    @SideOnly(Side.CLIENT)
    public boolean renderAsNormalBlock() {
        return false;
    }

    /**
     * Retrieves Minecraft's internal ID of the block from HexClientProxy.
     */
    @Override
    @SideOnly(Side.CLIENT)
    public int getRenderType() {
        return HexClientProxy.renderID[idBlock];
    }

    /**
     * Makes the block render on both passes.
     */
    @Override
    @SideOnly(Side.CLIENT)
    public boolean canRenderInPass(int pass) {
        // Save the current rendering pass of the block to HexClientProxy.
        HexClientProxy.renderPass[idBlock] = pass;
        return true;
    }

    /**
     * Makes the block render on both passes.
     */
    @Override
    @SideOnly(Side.CLIENT)
    public int getRenderBlockPass() {
        return 1;
    }

    /**
     * Makes the block properly interact with AO.
     */
    @Override
    @SideOnly(Side.CLIENT)
    public boolean isBlockNormalCube() {
        return true;
    }

    /**
     * Forces torch placement on top.
     */
    @Override
    public boolean canPlaceTorchOnTop(World world, int x, int y, int z) {
        return true;
    }

    /**
     * Forces torch placement on sides.
     */
    @Override
    public boolean isSideSolid(IBlockAccess world, int x, int y, int z, ForgeDirection side) {
        return true;
    }

    /**
     * Forces Redstone conduction.
     */
    @Override
    public boolean shouldCheckWeakPower(IBlockAccess world, int x, int y, int z, int side) {
        return true;
    }
}
